package cn.iocoder.yudao.module.order.service.orderSplit;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.order.vo.approval.OrderApprovalCancelApplyReqVO;
import cn.iocoder.yudao.module.order.vo.orderSplit.*;
import cn.iocoder.yudao.module.order.dal.dataobject.orderSplit.OrderSplitDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 订单拆单 Service 接口
 *
 * @author yanghao
 */
public interface OrderSplitService extends IService<OrderSplitDO> {

    /**
     * 创建订单拆单
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Boolean createSplit(@Valid OrderSplitCreateReqVO createReqVO);

    /**
     * 更新订单拆单
     *
     * @param updateReqVO 更新信息
     */
    void updateSplit(@Valid OrderSplitUpdateReqVO updateReqVO);

    /**
     * 重置拆单
     *
     * @param orderId 编号
     * @return
     */
    Boolean deleteAllSplit(Long orderId);

    /**
     * 删除订单拆单
     *
     * @param id 编号
     * @return
     */
    Boolean deleteSplit(Long id);

    /**
     * 拆单申请
     *
     * @param applyReqVO 申请请求
     * @return
     */
    boolean splitApply(OrderSplitApplyReqVO applyReqVO);

    /**
     * 拆单申请-回调
     *
     * @param approvalId
     * @param result
     * @return
     */
    void splitApplyCallback(String approvalId, Integer result);

    /**
     * 取消拆单及申请
     *
     * @param cancelApplyReqVO@return
     */
    boolean cancelSplitApply(OrderApprovalCancelApplyReqVO cancelApplyReqVO);

    /**
     * 获得订单拆单
     *
     * @param id 编号
     * @return 订单拆单
     */
    OrderSplitDO getSplit(Long id);

    /**
     * 根据订单号获得订单拆单列表
     *
     * @param orderId 订单号
     * @param lang
     * @return 订单拆单列表
     */
    List<OrderSplitBackVO> listByOrderId(Long orderId, Integer lang);

    /**
     * 获得订单拆单列表
     *
     * @param ids 编号
     * @return 订单拆单列表
     */
    List<OrderSplitDO> getSplitList(Collection<Long> ids);

    /**
     * 获得订单拆单分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 订单拆单分页
     */
    PageResult<OrderSplitDO> getSplitPage(OrderSplitQueryVO query, PageVO page);

    /**
     * 获得订单拆单列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 订单拆单列表
     */
    List<OrderSplitDO> getSplitList(OrderSplitQueryVO query);

    /**
     * 拆单取消
     *
     * @param orderId 订单ID
     */
    void splitCancelByOrderId(Long orderId);

    /**
     * 拆单审核成功
     *
     * @param orderSplitApplyDetailDto 订单
     */
    void splitSuccess(OrderSplitApplyDetailDto orderSplitApplyDetailDto, String auditResult);

    void removeByOrderId(Long orderId);

    /**
     * 撤销拆单申请详情
     *
     * @param orderId 订单id
     * @return
     */
    OrderSplitRevokeApplyDetailDto revokeSplitInfo(Long orderId);

    /**
     * 撤销拆单申请的审批详情
     * 
     * @param approvalId 审批业务id
     * @return
     */
    OrderSplitRevokeApplyDetailDto revokeSplitInfoByApprovalId(Long approvalId);

    /**
     * 撤销拆单申请
     *
     * @param applyReqVO 申请
     * @return
     */
    boolean revoke(OrderSplitApplyReqVO applyReqVO);


    /**
     * 取消撤销拆单申请
     *
     * @param cancelApplyReqVO 取消申请
     * @return
     */
    boolean cancelRevoke(OrderApprovalCancelApplyReqVO cancelApplyReqVO);


    /**
     * 撤销拆单申请-回调
     *
     * @param approvalId
     * @param result
     * @return
     */
    void revokeSplitApplyCallback(String approvalId, Integer result);
}
